﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Json;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using FluxJpeg.Core;
using System.IO;
using System.Windows.Printing;
using System.Runtime.InteropServices;




namespace SLComikeTagGen
{
    public partial class MainPage : UserControl
    {
        SaveFileDialog saveFileDialog;

        private string HallOptionsJson = @"
[
{id:'E1', Name:'東１'},
{id:'E2', Name:'東２'},
{id:'E3', Name:'東３'},
{id:'E4', Name:'東４'},
{id:'E5', Name:'東５'},
{id:'E6', Name:'東６'},
{id:'W1', Name:'西１'},
{id:'W2', Name:'西２'}
]";

        private string BGColorOptionsJson = @"
[
{id:'1', Name:'Day1', Text:'１日目－緑色', Background:'Green'},
{id:'2', Name:'Day2', Text:'２日目－黄色', Background:'Yellow'},
{id:'3', Name:'Day3', Text:'３日目－紫色', Background:'Violet'},
{id:'4', Name:'Transparent', Text:'透明', Background:'Transparent'}
]";


        public class HallOption
        {
            public string id {get; set;}
            public string Name {get; set;}

            public HallOption(string _id, string _name)
            {
                // TODO: Complete member initialization
                this.id = _id;
                this.Name = _name;
            }
        }

        public class BGColorOption
        {
            public string id { get; set; }
            public string Name {get; set;}
            public string Text { get; set; }
            public string Background { get; set; }

            public BGColorOption(string id, string Name, string Text, string Background)
            {
                this.id = id;
                this.Name = Name;
                this.Text = Text;
                this.Background = Background;
            }
        }

        public MainPage()
        {
            InitializeComponent();
            this.Loaded +=new RoutedEventHandler(MainPage_Loaded);

        }

        /// <summary>
        /// 初期化処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void  MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            saveFileDialog = new SaveFileDialog();
            saveFileDialog.DefaultExt = "jpg";
            saveFileDialog.Filter = "JPG Files|*.jpg";

            JsonArray HallOptionArray = (JsonArray)(JsonArray.Load(new StringReader(HallOptionsJson)));
            List<HallOption> HallOptionsList = new List<HallOption>();
            foreach(JsonValue i in HallOptionArray)
            {
                HallOptionsList.Add(new HallOption(i["id"], i["Name"]));
            }
            cbHall.ItemsSource = HallOptionsList;

            //JsonArray BGColorOptions = (JsonArray)(JsonArray.Load(new StringReader(BGColorOptionsJson)));
            //List<BGColorOption>  BGColorOptionsList = new List<BGColorOption>();
            //foreach(JsonValue i in BGColorOptions){
            //    BGColorOptionsList.Add(new BGColorOption(i["id"], i["Name"], i["Text"], i["Background"]));
            //}
            //cbBGColor.DataContext = BGColorOptionsList;


        }


        private void button1_Click(object sender, RoutedEventArgs e)
        {

            var w = TagPreview.ActualWidth;
            var h = TagPreview.ActualHeight;

            ScaleTransform tr = new ScaleTransform();
            tr.ScaleX = 2508 / w;
            tr.ScaleY = 1764 / h;

            //var wbmp = new WriteableBitmap(TagPreview);
            var wbmp = new WriteableBitmap(2508, 1764);
            wbmp.Render(TagPreview, tr);
            wbmp.Invalidate();

            bool? dialogResult = saveFileDialog.ShowDialog();

            if (dialogResult == true)
            {
                // WriteableBitmap を JPEGに変換して保存
                byte[] fileBytes  =CreateImageAsJpeg(wbmp, 95);

                // ユーザーのボタン操作によってファイル保存処理を
                // 行わないとセキュリティ例外
                using (Stream fs = (Stream)this.saveFileDialog.OpenFile())
                {
                    // ユーザーが指定したファイル名で保存
                    fs.Write(fileBytes, 0, fileBytes.Length);
                    fs.Close();
                }
            }
        }

        //http://d.hatena.ne.jp/ksasao/20100719/1279550667
        /// <summary>
        /// ビットマップをJpegのバイト列へ変換します
        /// </summary>
        /// <param name="bitmap">変換元画像</param>
        /// <param name="quality">圧縮品質(0:最高圧縮率～100:最高品質)</param>
        /// <returns>Jpegエンコードされたバイト列</returns>
        private byte[] CreateImageAsJpeg(WriteableBitmap bitmap, int quality)
        {
            int width = bitmap.PixelWidth;
            int height = bitmap.PixelHeight;
            int bands = 3;
            byte[][,] raster = new byte[bands][,];

            for (int i = 0; i < bands; i++)
            {
                raster[i] = new byte[width, height];
            }

            for (int row = 0; row < height; row++)
            {
                for (int column = 0; column < width; column++)
                {
                    int pixel = bitmap.Pixels[width * row + column];
                    raster[0][column, row] = (byte)(pixel >> 16);
                    raster[1][column, row] = (byte)(pixel >> 8);
                    raster[2][column, row] = (byte)pixel;
                }
            }

            ColorModel model = new ColorModel { colorspace = ColorSpace.RGB };
            FluxJpeg.Core.Image img = new FluxJpeg.Core.Image(model, raster);

            // JPEG形式でエンコード
            MemoryStream stream = new MemoryStream();
            FluxJpeg.Core.Encoder.JpegEncoder encoder
                = new FluxJpeg.Core.Encoder.JpegEncoder(img, quality, stream);
            encoder.Encode();

            // MemoryStream の先頭に移動
            stream.Seek(0, SeekOrigin.Begin);

            // MemoryStream の内容を配列にコピー
            byte[] binaryData = new Byte[stream.Length];
            long bytesRead = stream.Read(binaryData, 0, (int)stream.Length);
            return binaryData;
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            var printDoc = new PrintDocument();
            printDoc.PrintPage += OnPrintPage;
            printDoc.Print("SLComikeTagGet print");
        }

        private void OnPrintPage(object sender, PrintPageEventArgs args)
        {
            var w = TagPreview.ActualWidth;
            var h = TagPreview.ActualHeight;

            ScaleTransform tr = new ScaleTransform();
            //tr.ScaleX = ((182 / 25.4) * 96) / w;
            //tr.ScaleY = ((128 / 25.4) * 96) / h;

            tr.ScaleX = 688 / w;
            tr.ScaleY = 484 / h;

            var border = new Border();
            border.Padding = new Thickness(20);

            //no way to copy xaml object easily
            //workaround with creating bitmap

            var wbmp = new WriteableBitmap(TagPreview, tr);
            var image = new System.Windows.Controls.Image();
            image.Source = wbmp;
            image.VerticalAlignment = System.Windows.VerticalAlignment.Top;

            border.Child = image;

            //border.RenderTransform = tr;
            //border.InvalidateArrange();
            args.PageVisual = border;

            //image.Margin = margin;

            //TagPreview.RenderTransform = tr;
            //TagPreview.InvalidateArrange();
            //args.PageVisual = TagPreview;
            //args.PageVisual = image;
            
        }

        private void cbBGColor_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var selected = (sender as ComboBox).SelectedItem as ComboBoxItem;
            //MessageBox.Show(selected.Name);
            TagPreview.Background = selected.Background;
        }

        public void changeColor(Color color)
        {
            //MessageBox.Show(color.ToString());
            TagPreview.Background = new SolidColorBrush(color);

        }

        private void cbHall_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

        }

        private void tbxDate_TextChanged(object sender, TextChangedEventArgs e)
        {

        }

        private void tbxLocation_TextChanged(object sender, TextChangedEventArgs e)
        {

        }

        private void tbxCircleName_TextChanged(object sender, TextChangedEventArgs e)
        {

        }

        private void tbxOwnerName_TextChanged(object sender, TextChangedEventArgs e)
        {

        }

        private void btnCalendar_Click(object sender, RoutedEventArgs e)
        {

        }

        private void dpcDate_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {

        }

        private void btnChooseColor_Click(object sender, RoutedEventArgs e)
        {
            var chooser = new ColorChooser(this);
            chooser.Show();
        }
    }
}
